@@ -65,7 +65,7 @@ void srm_srmv2_mdfilestatus_delete(struct srmv2_mdfilestatus* mdfilestatus, int
 			free(mdfilestatus[i].checksum);
 			free(mdfilestatus[i].checksumtype);	
 			for(j=0; j < mdfilestatus[i].nbspacetokens;++j){
-				if(mdfilestatus !=NULL && mdfilestatus[i].spacetokens[i] ){
+				if(mdfilestatus && mdfilestatus[i].spacetokens[i] ){
 					free(mdfilestatus[i].spacetokens[i]);
 					mdfilestatus[i].spacetokens[i]=NULL;
 				}
