@@ -19,7 +19,7 @@
 #ifndef _SRM_TYPES_H
 #define _SRM_TYPES_H
 
-
+#include <errno.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <sys/stat.h>
@@ -42,6 +42,10 @@
 
 #define SRM_SIZE_MARGIN          1048576     // 1MB
 
+#ifndef ECOMM
+#define ECOMM EIO
+#endif
+
 typedef struct srm_context* srm_context_t;
 
 typedef struct srm_context_extension* srm_context_extension_t;
@@ -245,6 +249,12 @@ struct srm_mkdir_input
 	char 	*dir_name; // directory surl
 };
 
+struct srm_mv_input
+{
+    char    *from;
+    char    *to;
+};
+
 struct srm_preparetoget_input
 {
 	int nbfiles; // number of surls in the array
