@@ -24,6 +24,7 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <assert.h>
+#include <time.h>
 #include "srmv2Stub.h"
 #include "srm_soap.h"
 #include "srm_util.h"
@@ -164,8 +165,7 @@ struct soap * srm_soap_init_context_new(struct srm_context* c){
 
     if (c && c->ext && c->ext->keep_alive) {
         soap_handle = soap_new2(SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
-        soap_handle->bind_flags |= SO_REUSEADDR;
-        soap_handle->max_keep_alive = 50;
+        soap_handle->bind_flags |= SO_REUSEADDR;        
         soap_handle->accept_timeout = 0;
 	soap_handle->tcp_keep_alive = 1;
         soap_handle->socket_flags = MSG_NOSIGNAL;
@@ -201,15 +201,21 @@ void srm_soap_deinit(struct soap *soap)
 
 void srm_soap_free(struct soap *soap)
 {
+    const int srm_ifce_errno = errno;
 	soap_end (soap);
 	soap_done (soap);
 	soap_free(soap);
+    // override errno with srm-ifce specific value
+    // hack for globus openssl errno problems
+    errno = srm_ifce_errno;
 }
 
 
-const char * statuscode2errmsg (int statuscode)
+const char * statuscode2errmsg (unsigned int statuscode)
 {
-	return (srmv2_errmsg[statuscode]);
+    if(statuscode < (sizeof(srmv2_errmsg)/sizeof(char*)))
+        return (srmv2_errmsg[statuscode]);
+    return srmv2_errmsg[1];
 }
 
 int statuscode2errno (int statuscode)
@@ -238,6 +244,24 @@ int statuscode2errno (int statuscode)
 	}
 }
 
+int statuscode_and_msg_to_errno(int statuscode, const char** err_tab_msg){
+    int err_code = statuscode2errno(statuscode);
+    if(err_code == EINVAL && err_tab_msg != NULL){
+        // search for string pattern
+        char** err_tab = (char**) err_tab_msg;
+        while(*err_tab != NULL){
+            // search for pattern EEXIST, LCGUTIL-203 bug
+            if( strstr(*err_tab,"exists, overwite is not allowed") != NULL)
+                err_code = EEXIST;
+
+            ++err_tab;
+        }
+    }
+    return err_code;
+}
+
+
+
 void srm_errmsg (struct srm_context *context, const char *format, ...)
 {
 	va_list ap;
@@ -248,7 +272,10 @@ void srm_errmsg (struct srm_context *context, const char *format, ...)
 
 	va_start (ap, format);
      (void)asprintf (&actual_format, "%s\n", format);
-	if (actual_format == NULL) return;
+	if (actual_format == NULL){ 
+		va_end(ap);
+		return;
+	}
 
 	if (context->errbuf == NULL)
 		vfprintf (stderr, actual_format, ap);
@@ -256,6 +283,7 @@ void srm_errmsg (struct srm_context *context, const char *format, ...)
 		vsnprintf (context->errbuf, context->errbufsz, actual_format, ap);
 
 	free (actual_format);
+	va_end(ap);
 }
 int srm_soap_call_err(struct srm_context *context,struct soap *soap,const char *srmfunc)
 {
@@ -289,7 +317,8 @@ int srm_call_err(struct srm_context *context,struct srm2__TReturnStatus  *retsta
 			retstatus->statusCode != SRM_USCOREDONE &&
 			retstatus->statusCode != SRM_USCORETOO_USCOREMANY_USCORERESULTS)
 	{
-		result_errno = statuscode2errno (retstatus->statusCode);
+        const char * tab_err_msg[] = { retstatus->explanation, NULL };
+        result_errno = statuscode_and_msg_to_errno(retstatus->statusCode, tab_err_msg);
 		if (retstatus->explanation && retstatus->explanation[0])
 		{
 			srm_errmsg (context, "[%s][%s][%s] %s: %s",err_msg_begin,
@@ -1095,6 +1124,18 @@ int copy_mdfilestatuses(struct srm2__TReturnStatus *reqstatp,
 		{
 			(*statuses)[i].stat.st_mode |= repfs->pathDetailArray[i]->ownerPermission->mode << 6;
 		}
+		if (repfs->pathDetailArray[i]->createdAtTime)
+		{
+			struct tm createdAtTime;
+			strptime(repfs->pathDetailArray[i]->createdAtTime, "%Y-%m-%dT%T", &createdAtTime);
+			(*statuses)[i].stat.st_ctime = mktime(&createdAtTime);
+		}
+		if (repfs->pathDetailArray[i]->lastModificationTime)
+		{
+			struct tm modifiedTime;
+			strptime(repfs->pathDetailArray[i]->lastModificationTime, "%Y-%m-%dT%T", &modifiedTime);
+			(*statuses)[i].stat.st_mtime = mktime(&modifiedTime);
+		}
 		if (repfs->pathDetailArray[i]->type)
 		{
 			switch (*(repfs->pathDetailArray[i]->type))
