@@ -54,7 +54,7 @@ int srm_print_error_status(struct srm_context *context,struct srm2__TReturnStatu
 int srm_print_error_status_additional(struct srm_context *context,struct srm2__TReturnStatus *status,const char *srmfunc,char *additional_info);
 
 int statuscode2errno (int statuscode);
-const char * statuscode2errmsg (int statuscode);
+const char * statuscode2errmsg (unsigned int statuscode);
 
 void back_off_logic_init(struct srm_context *context,struct srm_internal_context *internal_context);
 srm_call_status back_off_logic(struct srm_context *context,const char *srmfunc,
