@@ -148,6 +148,20 @@ int srm_mkdir(struct srm_context *context,struct srm_mkdir_input *input)
 	}
 }
 
+int srm_mv(struct srm_context *context, struct srm_mv_input *input)
+{
+    switch (context->version)
+    {
+        case VERSION_2_2:
+            return srmv2_mv(context,input);
+        case VERSION_1:
+            // TODO
+            return (-1);
+        default:
+            return (-1);
+    }
+}
+
 int srm_release_files(struct srm_context *context,
 		struct srm_releasefiles_input *input, struct srmv2_filestatus **statuses)
 {
