@@ -16,50 +16,28 @@
  *
  * Authors: Todor Manev  IT-GT CERN
  */
-#include <errno.h>
-#include <assert.h>
-#include "gfal_srm_ifce.h"
+
+#include "srm_ifce_internal.h"
 #include "srm_soap.h"
-#include "srm_util.h"
-#include "srm_dependencies.h"
 
 
 
+int srmv2_put_status_estimated_wait_time(struct srm2__srmStatusOfPutRequestResponse_ *prepareToPutStatusRep){
+    if( prepareToPutStatusRep && prepareToPutStatusRep->srmStatusOfPutRequestResponse && prepareToPutStatusRep->srmStatusOfPutRequestResponse->arrayOfFileStatuses
+        && prepareToPutStatusRep->srmStatusOfPutRequestResponse->arrayOfFileStatuses->__sizestatusArray > 0
+        && prepareToPutStatusRep->srmStatusOfPutRequestResponse->arrayOfFileStatuses->statusArray){
+        struct srm2__TPutRequestFileStatus* status= *prepareToPutStatusRep->srmStatusOfPutRequestResponse->arrayOfFileStatuses->statusArray;
+        if(status && status->estimatedWaitTime){
+            return *status->estimatedWaitTime;
+        }
+    }
+    return -1;
+}
+
+
 int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 		struct srm_preparetoput_input *input,
-		struct srm_preparetoget_output *output,
-		struct srm_internal_context *internal_context);
-
-int srmv2_prepare_to_put_async_internal(struct srm_context *context,
-		struct srm_preparetoput_input *input,
-		struct srm_preparetoput_output *output,
-		struct srm_internal_context *internal_context);
-
-int srmv2_prepare_to_get_async_internal(struct srm_context *context,
-		struct srm_preparetoget_input *input,
-		struct srm_preparetoget_output *output,
-		struct srm_internal_context *internal_context);
-
-int srmv2_status_of_get_request_async_internal(struct srm_context *context,
-		struct srm_preparetoget_input *input,
-		struct srm_preparetoget_output *output,
-		struct srm_internal_context *internal_context);
-
-int srmv2_status_of_bring_online_async_internal (struct srm_context *context,
-		struct srm_bringonline_input *input,
-		struct srm_bringonline_output *output,
-		struct srm_internal_context *internal_context);
-
-int srmv2_bring_online_async_internal (struct srm_context *context,
-		struct srm_bringonline_input *input,
-		struct srm_bringonline_output *output,
-		struct srm_internal_context *internal_context);
-
-
-
-int srmv2_status_of_put_request_async_internal(struct srm_context *context,
-		struct srm_preparetoput_input *input,
-		struct srm_preparetoget_output *output,
+        struct srm_preparetoput_output *output,
 		struct srm_internal_context *internal_context)
 {
 	struct srm2__ArrayOfTPutRequestFileStatus *repfs;
@@ -86,7 +64,7 @@ int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 			srm_soap_free(soap);
 			return -1;
 		}
-		set_estimated_wait_time(internal_context, srep.srmStatusOfPutRequestResponse->remainingTotalRequestTime);
+        set_estimated_wait_time(internal_context, srmv2_put_status_estimated_wait_time(&srep));
 		// Check status and wait with back off logic if necessary(Internal_error)
 		internal_context->current_status = back_off_logic(context,srmfunc,internal_context,output->retstatus);
 
@@ -123,6 +101,10 @@ int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 											srmfunc);
 			}
 			break;
+        case srm_call_status_TIMEOUT:
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -133,6 +115,18 @@ int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 }
 
 
+int srmv2_put_estimated_wait_time(struct srm2__srmPrepareToPutResponse_ *prepareToPutRep){
+    if(prepareToPutRep && prepareToPutRep->srmPrepareToPutResponse && prepareToPutRep->srmPrepareToPutResponse->arrayOfFileStatuses
+            && prepareToPutRep->srmPrepareToPutResponse->arrayOfFileStatuses->__sizestatusArray > 0
+            && prepareToPutRep->srmPrepareToPutResponse->arrayOfFileStatuses->statusArray){
+       struct srm2__TPutRequestFileStatus * status = *prepareToPutRep->srmPrepareToPutResponse->arrayOfFileStatuses->statusArray;
+       if(status && status->estimatedWaitTime){
+           return *status->estimatedWaitTime;
+       }
+    }
+    return -1;
+}
+
 
 int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 		struct srm_preparetoput_input *input,
@@ -140,19 +134,16 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 		struct srm_internal_context *internal_context)
 {
 	int i;
-	int n;
 	int ret = 0;
 	struct srm2__srmPrepareToPutResponse_ rep;
 	struct srm2__ArrayOfTPutRequestFileStatus *repfs;
 	struct srm2__srmPrepareToPutRequest req;
-	struct srm2__TReturnStatus *reqstatp;
     struct soap* soap = srm_soap_init_context_new(context);
 	static enum srm2__TFileStorageType s_types[] = {VOLATILE, DURABLE, PERMANENT};
 	char *targetspacetoken;
 	const char srmfunc[] = "PrepareToPut";
 	struct srm_getbestspacetokens_input spacetokeninput;
 	SRM_LONG64 totalsize=0;
-	struct srm_mkdir_input mkdirinput;
 
 
 	memset (&req, 0, sizeof(req));
@@ -174,14 +165,18 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 		return (-1);
 	}
 
-	if (input->desiredpintime > 0)
-	{
-		req.desiredPinLifeTime = &input->desiredpintime;
-	}
-	if (context->timeout > 0)
-	{
-		req.desiredTotalRequestTime = &context->timeout ;
-	}
+    if(context->ext && context->ext->turl_timeout){ // advanced context -> turl timeout
+        req.desiredPinLifeTime = &(context->ext->turl_timeout);
+        req.desiredTotalRequestTime = &(context->ext->turl_timeout);
+    }else{
+        if (input->desiredpintime > 0)
+            req.desiredPinLifeTime = &input->desiredpintime;
+        else
+            req.desiredPinLifeTime = &context->timeout_ops;
+
+        if (context->timeout_ops > 0)
+            req.desiredTotalRequestTime = &context->timeout_ops;
+    }
 
 	req.desiredFileStorageType = &s_types[PERMANENT];
 	req.arrayOfFileRequests->__sizerequestArray = input->nbfiles;
@@ -283,7 +278,7 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 			srm_soap_free(soap);
 			return -1;
 		}
-		set_estimated_wait_time(internal_context, rep.srmPrepareToPutResponse->remainingTotalRequestTime);
+        set_estimated_wait_time(internal_context, srmv2_put_estimated_wait_time(&rep));
 		// Check status and wait with back off logic if necessary(Internal_error)
 		internal_context->current_status = back_off_logic(context,srmfunc,internal_context,output->retstatus);
 
@@ -335,6 +330,10 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 				}				
 			}
 			break;
+        case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -345,7 +344,18 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 	return ret;
 }
 
-
+int srmv2_get_estimated_wait_time(struct srm2__srmPrepareToGetResponse_ *prepareTogetResp){
+    if(prepareTogetResp && prepareTogetResp->srmPrepareToGetResponse
+       && prepareTogetResp->srmPrepareToGetResponse->arrayOfFileStatuses &&
+       prepareTogetResp->srmPrepareToGetResponse->arrayOfFileStatuses->statusArray > 0
+       && prepareTogetResp->srmPrepareToGetResponse->arrayOfFileStatuses->statusArray){
+       struct srm2__TGetRequestFileStatus* status=  *prepareTogetResp->srmPrepareToGetResponse->arrayOfFileStatuses->statusArray;
+       if(status && status->estimatedWaitTime ){
+           return *status->estimatedWaitTime ;
+       }
+    }
+    return -1;
+}
 
 int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 		struct srm_preparetoget_input *input,
@@ -354,14 +364,11 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 {
 	char *targetspacetoken;
 	int i;
-	int n;
 	int ret = 0;
 	struct srm2__srmPrepareToGetResponse_ rep;
 	struct srm2__ArrayOfTGetRequestFileStatus *repfs;
 	struct srm2__srmPrepareToGetRequest req;
-	struct srm2__TReturnStatus *reqstatp;
 	struct srm_getbestspacetokens_input spacetokeninput;
-	int canary = 0;
     struct soap* soap = srm_soap_init_context_new(context);
 	static enum srm2__TFileStorageType s_types[] = {VOLATILE, DURABLE, PERMANENT};
 	const char srmfunc[] = "PrepareToGet";
@@ -404,14 +411,19 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 		}
 	}
 
-	if (input->desiredpintime > 0)
-	{
-		req.desiredPinLifeTime = &input->desiredpintime;
-	}
-	if (context->timeout > 0)
-	{
-		req.desiredTotalRequestTime = &context->timeout;
-	}
+
+    if(context->ext && context->ext->turl_timeout){ // advanced context -> turl timeout
+        req.desiredPinLifeTime = &(context->ext->turl_timeout);
+        req.desiredTotalRequestTime = &(context->ext->turl_timeout);
+    }else{
+        if (input->desiredpintime > 0)
+            req.desiredPinLifeTime = &input->desiredpintime;
+        else
+            req.desiredPinLifeTime = &context->timeout;
+
+        if (context->timeout_ops > 0)
+            req.desiredTotalRequestTime = &context->timeout_ops;
+    }
 
 	req.desiredFileStorageType = &s_types[PERMANENT];
 	req.arrayOfFileRequests->__sizerequestArray = input->nbfiles;
@@ -455,7 +467,7 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 			srm_soap_free(soap);
 			return -1;
 		}
-		set_estimated_wait_time(internal_context, rep.srmPrepareToGetResponse->remainingTotalRequestTime);
+        set_estimated_wait_time(internal_context, srmv2_get_estimated_wait_time(&rep));
 		// Check status and wait with back off logic if necessary(Internal_error)
 		internal_context->current_status = back_off_logic(context,srmfunc,internal_context,output->retstatus);
 
@@ -497,12 +509,20 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 			{
 				errno = 0;
 				internal_context->current_status = srm_call_status_SUCCESS;
+                if(rep.srmPrepareToGetResponse->requestToken != NULL){
+                    copy_string(&output->token,rep.srmPrepareToGetResponse->requestToken);
+                }
 				ret = copy_pinfilestatuses_get(output->retstatus,
 											&output->filestatuses,
 											repfs,
 											srmfunc);
+
 			}
 			break;
+        case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -513,6 +533,20 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 	return ret;
 }
 
+
+int srmv2_get_status_estimated_wait_time(struct srm2__srmStatusOfGetRequestResponse_ *prepareTogetRespstatus){
+    if(prepareTogetRespstatus && prepareTogetRespstatus->srmStatusOfGetRequestResponse
+            && prepareTogetRespstatus->srmStatusOfGetRequestResponse->arrayOfFileStatuses
+            && prepareTogetRespstatus->srmStatusOfGetRequestResponse->arrayOfFileStatuses->__sizestatusArray > 0
+            && prepareTogetRespstatus->srmStatusOfGetRequestResponse->arrayOfFileStatuses->statusArray){
+        struct srm2__TGetRequestFileStatus* status = *prepareTogetRespstatus->srmStatusOfGetRequestResponse->arrayOfFileStatuses->statusArray;
+        if(status && status->estimatedWaitTime)
+            return *status->estimatedWaitTime;
+    }
+    return -1;
+}
+
+
 int srmv2_status_of_get_request_async_internal(struct srm_context *context,
 		struct srm_preparetoget_input *input,
 		struct srm_preparetoget_output *output,
@@ -523,7 +557,7 @@ int srmv2_status_of_get_request_async_internal(struct srm_context *context,
     struct soap* soap = srm_soap_init_context_new(context);
 	struct srm2__srmStatusOfGetRequestResponse_ srep;
 	struct srm2__srmStatusOfGetRequestRequest sreq;
-	const char srmfunc[] = "StatusOfPutRequest";
+    const char srmfunc[] = "StatusOfGetRequest";
 
 
 	memset (&sreq, 0, sizeof(sreq));
@@ -543,7 +577,7 @@ int srmv2_status_of_get_request_async_internal(struct srm_context *context,
 			srm_soap_free(soap);
 			return -1;
 		}
-		set_estimated_wait_time(internal_context, srep.srmStatusOfGetRequestResponse->remainingTotalRequestTime);
+        set_estimated_wait_time(internal_context, srmv2_get_status_estimated_wait_time(&srep));
 		// Check status and wait with back off logic if necessary(Internal_error)
 		internal_context->current_status = back_off_logic(context,srmfunc,internal_context,output->retstatus);
 
@@ -584,6 +618,10 @@ int srmv2_status_of_get_request_async_internal(struct srm_context *context,
 											srmfunc);
 			}
 			break;
+        case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -597,13 +635,12 @@ int srmv2_status_of_get_request_async_internal(struct srm_context *context,
 int srmv2_put_done(struct srm_context *context,
 		struct srm_putdone_input *input, struct srmv2_filestatus **statuses)
 {
-	srm_call_status current_status;
 	struct srm_internal_context internal_context;
 	int ret=0;
 	struct srm2__srmPutDoneResponse_ rep;
+    struct srm2__TReturnStatus *reqstatp;
 	struct srm2__ArrayOfTSURLReturnStatus *repfs;
 	struct srm2__srmPutDoneRequest req;
-	struct srm2__TReturnStatus *reqstatp;
     struct soap* soap = srm_soap_init_context_new(context);
 	const char srmfunc[] = "PutDone";
 
@@ -646,18 +683,27 @@ int srmv2_put_done(struct srm_context *context,
 
 	}while (internal_context.current_status == srm_call_status_INTERNAL_ERROR);
 
+    switch(internal_context.current_status){
 
-	if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray)
-	{
-		errno = srm_call_err(context,reqstatp,srmfunc);
-		srm_soap_free(soap);
-		return (-1);
-	}else
-	{
-		errno = 0;
-		internal_context.current_status = srm_call_status_SUCCESS;
-		ret = copy_filestatuses(reqstatp,statuses,repfs,srmfunc);
-	}
+         case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
+        default:
+            if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray)
+            {
+                errno = srm_call_err(context,reqstatp,srmfunc);
+                srm_soap_free(soap);
+                return (-1);
+            }else
+            {
+                errno = 0;
+                internal_context.current_status = srm_call_status_SUCCESS;
+                ret = copy_filestatuses(reqstatp,statuses,repfs,srmfunc);
+            }
+            break;
+
+    }
 
 	srm_soap_free(soap);
 	return (ret);
@@ -665,7 +711,6 @@ int srmv2_put_done(struct srm_context *context,
 int srmv2_release_files(struct srm_context *context,
 		struct srm_releasefiles_input *input, struct srmv2_filestatus **statuses)
 {
-	srm_call_status current_status;
 	struct srm_internal_context internal_context;
 	int ret;
 	struct srm2__srmReleaseFilesResponse_ rep;
@@ -716,39 +761,59 @@ int srmv2_release_files(struct srm_context *context,
 
 	}while (internal_context.current_status == srm_call_status_INTERNAL_ERROR);
 
-	if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray)
-	{
-		errno = srm_call_err(context,reqstatp,srmfunc);
-		srm_soap_free(soap);
-		return (-1);
-	}else
-	{
-		errno = 0;
-		internal_context.current_status = srm_call_status_SUCCESS;
-		ret = copy_filestatuses(reqstatp,statuses,repfs,srmfunc);
-	}
+    switch(internal_context.current_status){
+
+         case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
+        default:
+            if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray)
+            {
+                errno = srm_call_err(context,reqstatp,srmfunc);
+                srm_soap_free(soap);
+                return (-1);
+            }else
+            {
+                errno = 0;
+                internal_context.current_status = srm_call_status_SUCCESS;
+                ret = copy_filestatuses(reqstatp,statuses,repfs,srmfunc);
+            }
+            break;
+    }
 
 	srm_soap_free(soap);
 	return (ret);
 }
 
 
+int srmv2_bringonline_estimated_wait_time(struct srm2__srmBringOnlineResponse_ *bringOnlineRep){
+    if( bringOnlineRep && bringOnlineRep->srmBringOnlineResponse && bringOnlineRep->srmBringOnlineResponse->arrayOfFileStatuses
+            && bringOnlineRep->srmBringOnlineResponse->arrayOfFileStatuses->__sizestatusArray > 0
+            && bringOnlineRep->srmBringOnlineResponse->arrayOfFileStatuses->statusArray){
+        struct  srm2__TBringOnlineRequestFileStatus* status =  *bringOnlineRep->srmBringOnlineResponse->arrayOfFileStatuses->statusArray;
+        if(status && status->estimatedWaitTime){
+            return *status->estimatedWaitTime;
+        }
+    }
+    return -1;
+}
+
+
+
 int srmv2_bring_online_async_internal (struct srm_context *context,
 		struct srm_bringonline_input *input,
 		struct srm_bringonline_output *output,
 		struct srm_internal_context *internal_context)
 {
-	srm_call_status current_status;
 	int ret,i;
 	struct srm_getbestspacetokens_input spacetokeninput;
 	struct srm2__srmBringOnlineResponse_ rep;
 	struct srm2__ArrayOfTBringOnlineRequestFileStatus *repfs;
 	struct srm2__srmBringOnlineRequest req;
-	struct srm2__TReturnStatus *reqstatp;
     struct soap* soap = srm_soap_init_context_new(context);
 	static enum srm2__TFileStorageType s_types[] = {VOLATILE, DURABLE, PERMANENT};
 	char *targetspacetoken;
-	int nbproto = 0;
 	const char srmfunc[] = "BringOnline";
 
 
@@ -836,7 +901,7 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 			return -1;
 		}
 
-		set_estimated_wait_time(internal_context, rep.srmBringOnlineResponse->remainingTotalRequestTime);
+        set_estimated_wait_time(internal_context, srmv2_bringonline_estimated_wait_time(&rep));
 		// Check status and wait with back off logic if necessary(Internal_error)
 		internal_context->current_status = back_off_logic(context,srmfunc,internal_context,output->retstatus );
 
@@ -883,6 +948,10 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 											srmfunc);
 			}
 			break;
+        case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+           errno = ETIMEDOUT;
+           ret = -1;
+           break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -892,17 +961,29 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 	srm_soap_free(soap);
 	return ret;
 }
+
+
+
+int srmv2_bringonline_status_estimated_wait_time(struct srm2__srmStatusOfBringOnlineRequestResponse_ *bringOnlineStatusRep){
+    if( bringOnlineStatusRep && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse->arrayOfFileStatuses
+         && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse->arrayOfFileStatuses->__sizestatusArray > 0
+         && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse->arrayOfFileStatuses->statusArray){
+        struct srm2__TBringOnlineRequestFileStatus* status = *bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse->arrayOfFileStatuses->statusArray;
+        if(status && status->estimatedWaitTime){
+            return *status->estimatedWaitTime;
+        }
+    }
+    return -1;
+}
+
+
 int srmv2_status_of_bring_online_async_internal (struct srm_context *context,
 		struct srm_bringonline_input *input,
 		struct srm_bringonline_output *output,
 		struct srm_internal_context *internal_context)
 {
-	int sav_errno = 0;
-	int i = 0;
-	int n;
 	int ret;
 	struct srm2__ArrayOfTBringOnlineRequestFileStatus *repfs;
-	struct srm2__TReturnStatus *reqstatp;
     struct soap* soap = srm_soap_init_context_new(context);
 	struct srm2__srmStatusOfBringOnlineRequestResponse_ srep;
 	struct srm2__srmStatusOfBringOnlineRequestRequest sreq;
@@ -928,7 +1009,7 @@ int srmv2_status_of_bring_online_async_internal (struct srm_context *context,
 			srm_soap_free(soap);
 			return -1;
 		}
-		set_estimated_wait_time(internal_context, srep.srmStatusOfBringOnlineRequestResponse->remainingTotalRequestTime);
+        set_estimated_wait_time(internal_context, srmv2_bringonline_status_estimated_wait_time(&srep));
 		// Check status and wait with back off logic if necessary(Internal_error)
 		internal_context->current_status = back_off_logic(context,srmfunc,internal_context,output->retstatus);
 
@@ -969,6 +1050,10 @@ int srmv2_status_of_bring_online_async_internal (struct srm_context *context,
 												srmfunc);
 			}
 			break;
+        case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -1047,6 +1132,10 @@ int srmv2_abort_files(struct srm_context *context,
 				ret = copy_filestatuses(reqstatp,statuses,repfs,srmfunc);
 			}
 			break;
+        case srm_call_status_TIMEOUT: // add timeout management for backoff logic
+           errno = ETIMEDOUT;
+           ret = -1;
+           break;
 		default:
 			ret = -1;
 			break;
